$ifndef _HPWM14E_INC_
$define _HPWM14E_INC_
'
'*************************************************************************************************************
'* This source code is provided 'as is' and any express or implied warranties, including,                    *
'* but not limited to, the implied warranties of merchantability and fitness for a particular                *
'* purpose are disclaimed.                                                                                   *
'*                                                                                                           *
'* In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,   *
'* special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute   *
'* goods or services; loss of use, data, or profits;  or business interruption) however caused and on any    *
'* theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)   *
'* arising in any way out of the use of this source code, even if advised of the possibility of such damage. *
'*                                                                                                           *
'* Any views and conclusions contained in the source code and any documentation are those of the author's    *
'* and should not be interpreted as representing official policies,                                          *
'* either expressed or implied, of Crownhill Associates Ltd.                                                 *
'*************************************************************************************************************
'
' Replacement routine for the compiler's HPWM command written in Proton BASIC.
' For Enhanced 14-bit core devices only using compiler version 3.5.5.5 onwards.
'
' Written by Les Johnson 30-01-13
'   
    #disable HPWM               ' Disable the compiler's HPWM library routine
    #enable Divide, Multiply    ' Enable the compiler's 16-bit multiply and divide routines
'
' Create some compiler system variables
'   
    Dim GEN As Byte System
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP1 As Byte System
    Dim PP1H As Byte System
    Dim PP2 As Byte System
    Dim PP2H As Byte System
    Dim PP3 As Byte System
    Dim PP3H As Byte System
    Dim PP4H As Byte System
    
    Dim wPP0 As PP0.Word
    Dim wPP1 As PP1.Word
    Dim wPP2 As PP2.Word
    Dim wPP3 As PP3.Word
    
    Dim HPWM_wFreq As wPP1
    Dim HPWM_bDuty As GEN
    Dim HPWM_bChannel As PP4H
'    
' Alias some SFRs as 16-bit    
'    
    Dim HPWM_wFSR0 As FSR0L.Word
    Dim HPWM_wCCPR1 As CCPR1L.Word
    Dim HPWM_wT2CON As T2CON.Word
    
$ifdef _CCPR2L
    Dim HPWM_wCCPR2 As CCPR2L.Word
$endif
$ifdef _CCPR3L
    Dim HPWM_wCCPR3 As CCPR3L.Word
$endif 
$ifdef _CCPR4L
    Dim HPWM_wCCPR4 As CCPR4L.Word
$endif 
$ifdef _CCPR5L
    Dim HPWM_wCCPR5 As CCPR5L.Word
$endif
 
#ifdef HPWM#REQ   
'------------------------------------------------------------------------------------    
    GoTo _HPWM_Main_                                    ' Jump over the subroutine
'------------------------------------------------------------------------------------
' HPWM      : Output pulse width modulation using the CCP peripherals                                                             
' Input     : WREG = channel (1 - 5)                            
'           : PP1/PP1H = 16-bit frequency                                 
'           : GEN = 8-bit duty cycle                                
' Output    : None                                                              
' Notes     : None                                                 
'
__hpwm_:   
    HPWM_bChannel = WREG                                ' Save the channel
    If HPWM_bChannel = 1 Then                           ' Do we want CCP1?
        HPWM_wFSR0 = VarPtr HPWM_wCCPR1                 ' Yes. So point FSR0L\H at CCPR1L\H    
        Output __CCP1_Pin                               ' Set CCP1 to output
        GoTo _hpwm_CalcFrequency
    EndIf

    $ifdef _CCPR2L                                      ' Does the device contain the CCPR2L SFR?
        If HPWM_bChannel = 2 Then                       ' Yes. So do we want CCP2?
            HPWM_wFSR0 = VarPtr HPWM_wCCPR2             ' Yes. So point FSR0L\H at CCPR2L\H        
            #ifSym __CCP2_Pin                           ' Has the CCP2_Pin declare been used in the program?
            Output __CCP2_Pin                           ' Yes. So set CCP2 to output
            #endIfSym   ' __CCP2_Pin
            GoTo _hpwm_CalcFrequency
        EndIf
    $endif  ' _CCPR2L
    
    $ifdef _CCPR3L                                      ' Does the device contain the CCPR3L SFR?
        If HPWM_bChannel = 3 Then                       ' Yes. So do we want CCP3?
            HPWM_wFSR0 = VarPtr HPWM_wCCPR3             ' Yes. So point FSR0L\H at CCPR3L\H
            #ifSym __CCP3_Pin                           ' Has the CCP3_Pin declare been used in the program?
            Output __CCP3_Pin                           ' Yes. So set CCP3 to output
            #endIfSym   ' __CCP3_Pin
            GoTo _hpwm_CalcFrequency
        EndIf
    $endif  ' _CCPR3L
    
    $ifdef _CCPR4L                                      ' Does the device contain the CCPR4L SFR?
        If HPWM_bChannel = 4 Then                       ' Yes. So do we want CCP4?
            HPWM_wFSR0 = VarPtr HPWM_wCCPR4             ' Yes. So point FSR0L\H at CCPR4L\H        
            #ifSym __CCP4_Pin                           ' Has the CCP4_Pin declare been used in the program?
            Output __CCP4_Pin                           ' Yes. So set CCP4 to output
            #endIfSym ' __CCP4_Pin
            GoTo _hpwm_CalcFrequency
        EndIf
    $endif  ' _CCPR4L
    
    $ifdef _CCPR5L                                      ' Does the device contain the CCPR5L SFR?
        If HPWM_bChannel = 5 Then                       ' Yes. So do we want CCP5?
            HPWM_wFSR0 = VarPtr HPWM_wCCPR5             ' Yes. So point FSR0L\H at CCPR5L\H
            #ifSym __CCP5_Pin                           ' Has the CCP5_Pin declare been used in the program?
            Output __CCP5_Pin                           ' Yes. So set CCP5 to output
            #endIfSym ' __CCP5_Pin
        EndIf
    $else   ' _CCPR5L
        GoTo _hpwm_exit                                 ' It must be an Invalid channel so exit
    $endif  ' _CCPR5L
'
' PS = (fosc / 1024) / required frequency
' PRx = ((fosc / 4 / PS) / required frequency) - 1
'
_hpwm_CalcFrequency: 
    wPP0 = (_xtal * 1000000 / 4) & $FFFF
    wPP2 = ((_xtal * 1000000 / 4) >> 16) & $FFFF
    Call __divide_int_u1616_                            ' (Fosc / 4) / required frequency
    T2CON = T2CON & $FC                                 ' Clear the T2CON PS bits
    WREG = PP0H                                         ' Get the PS value into WREG
    If STATUSbits_Z = 0 Then                            ' PS = 1 so start the timer    
        Set T2CONbits_T2CKPS0                           ' Otherwise... PS = 4    
        Ror wPP0                                        ' \ 
        Ror wPP0                                        ' / PR2 / 4
        WREG = WREG & $FC                               ' Check for PS >= 4
        If STATUSbits_Z = 0 Then                        ' PS = 4 so start the timer    
            Clear T2CONbits_T2CKPS0                     ' Set for PS = 16
            Set T2CONbits_T2CKPS1
            Ror wPP0                                    ' \ 
            Ror wPP0                                    ' / PR2 / 4
            WREG = WREG & $F0                           ' Check for PS >= 16
            If STATUSbits_Z = 0 Then                    ' PS = 16 so start the timer   
                Set T2CONbits_T2CKPS0                   ' Set PS = 64
                Ror wPP0                                ' \ 
                Ror wPP0                                ' / PR2 / 4
            EndIf
        EndIf
    EndIf 
    Set T2CONbits_TMR2ON                                ' Start the timer
    PR2 = PP0 - 1                                       ' Load PR2
'
' CCP = ((PR2 + 1) * 4) */ duty cycle
' Result is ((PR2 + 1) * duty cycle) / 64
'
    If HPWM_bDuty = 255 Then                            ' Is the duty cycle 255?
        wPP2 = $FFFF                                    ' Yes. So set full on
    Else                                                ' Otherwise...
        wPP2 = HPWM_bDuty                               ' Preset for 256 * duty cycle
        wPP3 = HPWM_bDuty                               ' Prepare for the multiply
        WREG = PR2 + 1                                  ' WREG = PR2 + 1
        If STATUSbits_Z = 0 Then                        ' Does PR2 + 1 = 256?
            HPWM_wFreq = WREG                           ' No. So...
            Call __multiply_u1616_
        EndIf
    EndIf    
    WREG = PP2H                                         ' Load the top 8 CCP bits
    Movwi INDF0++                                       ' Load CCPRL and increment
    Moviw ++INDF0                                       ' Increment and read the current CCPCON value into WREG
    WREG = WREG & $CF                                   ' Clear the bottom 2 CCP bits
    If PP2.7 = 1 Then
        Set WREG.5                                      ' Set bit-5 of CCPCON
    EndIf
    If PP2.6 = 1 Then
        Set WREG.4                                      ' Set bit-4 of CCPCON
    EndIf
    WREG = WREG | $0C                                   ' Enable PWM mode
    INDF0 = WREG                                        ' Set CCPCON    
_hpwm_exit:
    Return

_HPWM_Main_:
#endif  ' HPWM#REQ
$endif  ' _HPWM14E_INC_



